{ config, lib, pkgs, inputs, ... }:

let
  cfg = config.my.programs.nvim;
  inherit (lib) mkEnableOption mkIf readFile;
in
{
  imports = lib.my.listModulesRec ./plugins;

  options.my.programs.nvim.enable = mkEnableOption "nvim";

  config = mkIf cfg.enable {
    home-manager.users.moritz = {
      xdg.configFile."nvim/snippets" = {
        recursive = true;
        source = ./plugins/snippets;
      };

      programs.neovim = {
        enable = true;
        package = inputs.neovim-nightly-overlay.packages.${pkgs.system}.default;
        vimAlias = true;
        vimdiffAlias = true;
        withNodeJs = true;
        withPython3 = true;
        extraPackages = with pkgs;
          [
            alejandra
            checkmake
            codespell
            deadnix
            dotenv-linter
            fish
            jq
            nil
            nixd
            nixpkgs-fmt
            nodePackages.bash-language-server
            python3Packages.python-lsp-server
            python3Packages.ruff-lsp
            shellcheck
            shfmt
            stable.yamlfix
            statix
            taplo
            yamllint
          ];
        extraLuaConfig = readFile ./options.lua;
        lazy.enable = true;
      };
    };
  };
}
